@@ -1,7 +1,7 @@
 /*
  * Main include file for Mathomatic.  Can be edited.
  *
- * Copyright (C) 1987-2011 George Gesslein II.
+ * Copyright (C) 1987-2012 George Gesslein II.
  
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -17,7 +17,7 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
-The chief copyright holder can be contacted at gesslein@linux.com, or
+The chief copyright holder can be contacted at gesslein@mathomatic.org, or
 George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
@@ -56,7 +56,7 @@ George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
 #define	PROMPT_STR	"-> "				/* user interface main prompt strings, preceded by the current equation number */
 #define	HTML_PROMPT_STR	"&minus;&gt; "			/* main prompt in HTML output mode, should be same number of columns as above */
 
-#define	MAX_CMD_LEN	min((max(PATH_MAX, 512)), 32768)	/* maximum command line length (not expression input), also max filename length */
+#define	MAX_CMD_LEN	min((max(PATH_MAX, 1024)), 16384)	/* maximum command line length (not expression input), also max filename length */
 #define	MAX_PROMPT_LEN	STANDARD_SCREEN_COLUMNS		/* maximum length of prompts */
 
 /*
@@ -170,3 +170,6 @@ enum language_list {
 #define	side_debug(level, p1, n1)	list_debug(level, p1, n1, NULL, 0)
 #define debug_string(level, str)	{ if (debug_level >= (level)) fprintf(gfp, "%s\n", str); }
 #endif
+
+/* The correct way to determine if equation number "en" is not in use. */
+#define empty_equation_space(en)	((en) < 0 || (en) >= n_equations || n_lhs[en] <= 0)
