@@ -1,7 +1,7 @@
 /*
  * Mathomatic commands that don't belong anywhere else.
  *
- * Copyright (C) 1987-2011 George Gesslein II.
+ * Copyright (C) 1987-2012 George Gesslein II.
  
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -17,7 +17,7 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
-The chief copyright holder can be contacted at gesslein@linux.com, or
+The chief copyright holder can be contacted at gesslein@mathomatic.org, or
 George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
@@ -37,7 +37,7 @@ static int complex_func(char *cp, int imag_flag);
 static int elim_sub(int i, long v);
 
 /* Global variables for the optimize command. */
-static int	opt_en[N_EQUATIONS];
+static int	opt_en[N_EQUATIONS+1];
 static int	last_temp_var = 0;
 
 #if	SHELL_OUT
@@ -56,7 +56,7 @@ char	*cp;	/* the command line argument */
 	int		start, stop;
 	int		first_time = true;
 	int		cl1_len = 0, cl2_len = 0, len;
-	char		*cp1, cl[MAX_CMD_LEN], cl1[MAX_CMD_LEN], cl2[MAX_CMD_LEN], *exp_str;
+	char		*cp1, cl[16384+MAX_CMD_LEN], cl1[16384], cl2[MAX_CMD_LEN], *exp_str;
 	long		v, vx;			/* Mathomatic variables */
 	token_type	*equation;
 	int		*np;
@@ -146,9 +146,8 @@ char	*cp;	/* the command line argument */
 		return false;
 	}
 	if (shell_out(cl)) {
-		error(_("Error trying to run gnuplot."));
+		error(_("Possible error running gnuplot."));
 		printf(_("Shell command line = \"%s\".\n"), cl);
-		return false;
 #if	!SILENT
 	} else if (debug_level > 0) {
 		fprintf(gfp, _("Shell command line = \"%s\".\n"), cl);
@@ -165,7 +164,7 @@ int
 version_cmd(cp)
 char	*cp;	/* the command line argument */
 {
-	int		rv;	/* return value */
+	int	rv = true;	/* return value */
 
 	if (extra_characters(cp))	/* Make sure nothing else is on the command line. */
 		return false;
@@ -173,17 +172,18 @@ char	*cp;	/* the command line argument */
 	free_result_str();
 	result_str = strdup(VERSION);
 #endif
+#if	!SILENT || !LIBRARY
 	rv = version_report();
+#endif
 
+#if	!SILENT
 	fprintf(gfp, _("\nMathomatic is GNU LGPL version 2.1 licensed software,\n"));
 	fprintf(gfp, _("meaning it is free software that comes with no warranty.\n"));
 	fprintf(gfp, _("Type \"help license\" for the copyright and license.\n"));
 
 	fprintf(gfp, _("\nDocumentation, reference cards, and the newest version of Mathomatic\n"));
 	fprintf(gfp, _("are always available at the Mathomatic website: www.mathomatic.org\n"));
-	fprintf(gfp, _("Feedback, questions, and contributions in truth are always welcomed.\n"));
-	fprintf(gfp, _("Errors in the code or documentation are quickly analyzed and fixed\n"));
-	fprintf(gfp, _("as long as the chief author (George Gesslein II) is able to do so.\n"));
+#endif
 	return rv;
 }
 
@@ -317,6 +317,7 @@ char	*cp;
 		cp = skip_param(cp);
 	}
 	if (!get_range(&cp, &start, &stop)) {
+		warning(_("No equations to solve."));
 		return false;
 	}
 	i = next_espace();
@@ -788,8 +789,13 @@ int		*np;
 							;
 						simp_loop(rhs[i2], &n_rhs[i2]);
 						simp_loop(equation, np);
-						for (i = 0; opt_en[i] >= 0; i++)
-							;
+						for (i = 0;; i++) {
+							if (i >= N_EQUATIONS) {
+								error_bug("Too many optimized equations.");
+							}
+							if (opt_en[i] < 0)
+								break;
+						}
 						opt_en[i] = i2;
 						opt_en[i+1] = -1;
 						return true;
@@ -810,7 +816,7 @@ char	*cp;
 {
 	int	i, j, k, i1;
 	int	start, stop;
-	int	rv = false, flag;
+	int	rv = false, flag, skip_flag;
 	int	start_en;
 	int	diff_sign;
 
@@ -846,8 +852,8 @@ char	*cp;
 			flag = false;
 			simp_equation(i);
 			for (j = 0; opt_en[j] >= 0; j++) {
-				simp_equation(opt_en[j]);
 				if (i != opt_en[j]) {
+					simp_equation(opt_en[j]);
 					while (find_more(lhs[i], &n_lhs[i], opt_en[j]))
 						flag = true;
 					while (find_more(rhs[i], &n_rhs[i], opt_en[j]))
@@ -886,7 +892,29 @@ char	*cp;
 			if (se_compare(lhs[i], n_lhs[i], rhs[i], n_rhs[i], &diff_sign) && !diff_sign) {
 				n_lhs[i] = 0;
 				n_rhs[i] = 0;
-			} else
+			}
+		}
+	}
+	if (rv) {
+		for (i = start; i <= stop; i++) {
+			if (n_lhs[i] == 0)
+				continue;
+			skip_flag = false;
+			do {
+				flag = false;
+				simp_equation(i);
+				for (j = 0; opt_en[j] >= 0; j++) {
+					if (i != opt_en[j]) {
+						simp_equation(opt_en[j]);
+						while (find_more(lhs[i], &n_lhs[i], opt_en[j]))
+							flag = true;
+						while (find_more(rhs[i], &n_rhs[i], opt_en[j]))
+							flag = true;
+					} else
+						skip_flag = true;
+				}
+			} while (flag);
+			if (!skip_flag)
 				list_sub(i);
 		}
 	}
@@ -906,8 +934,9 @@ char	*cp;
 {
 	int	start, stop;
 	int	k;
-	char	*cp1;
+	char	*cp1, *cp_start;
 
+	cp_start = cp;
 	if (!readline_enabled) {
 		error(_("Readline is currently turned off."));
 		return false;
@@ -915,11 +944,10 @@ char	*cp;
 	do {
 		cp1 = cp;
 		if (!get_range(&cp, &start, &stop)) {
-			return false;
+			goto push_text;
 		}
 		if (*cp && cp == cp1) {
-			error(_("Invalid argument."));
-			return false;
+			goto push_text;
 		}
 		for (k = start; k <= stop; k++) {
 			if (n_lhs[k]) {
@@ -932,6 +960,15 @@ char	*cp;
 	} while (*cp);
 	debug_string(0, _("Expression pushed.  Press the UP key to access."));
 	return true;
+
+push_text:
+	if (*cp_start) {
+		add_history(cp_start);
+		last_history_string = NULL;
+		debug_string(0, _("Text string pushed.  Press the UP key to access."));
+		return true;
+	}
+	return false;
 }
 
 /*
@@ -970,9 +1007,13 @@ FILE	*ofp;	/* output file pointer */
 #if	!SECURE
 	char	buf[MAX_CMD_LEN];
 
-	if (security_level < 3 && ofp && getcwd(buf, sizeof(buf))) {
-		fprintf(ofp, "directory = %s\n", buf);
-		return true;
+	if (security_level < 3 && ofp) {
+		if (getcwd(buf, sizeof(buf))) {
+			fprintf(ofp, "directory = %s\n", buf);
+			return true;
+		} else {
+			perror(NULL);
+		}
 	}
 #endif
 	return false;
@@ -1159,35 +1200,32 @@ char	*cp;
 	char	*cp1, *option_string;
 
 try_next_param:
-	cp = skip_space(cp);
+	cp = skip_comma_space(cp);
 	if (*cp == '\0') {
 		return true;
 	}
-	if (strncasecmp(cp, "dir", 3) == 0) {
+	if (strncasecmp(cp, "directory", 3) == 0) {
 		cp = skip_param(cp);
-#if	!SECURE && !LIBRARY
+#if	!SECURE
 		if (security_level < 3) {
 	 		if (*cp == '\0') {
 				cp1 = getenv("HOME");
-				if (cp1) {
-					cp = cp1;
+				if (cp1 == NULL) {
+					error(_("HOME environment variable not set."));
+					return false;
 				}
+				cp = cp1;
 			}
 			if (chdir(cp)) {
+				perror(cp);
 				error(_("Error changing directory."));
 				return false;
 			}
-			output_current_directory(stdout);
-			return true;
+			return output_current_directory(stdout);
 		}
 #endif
-#if	LIBRARY
-		error(_("Option disabled."));
-		return false;
-#else
 		error(_("Option disabled by security level."));
 		return false;
-#endif
 	}
 	negate = skip_no(&cp);
 	option_string = cp;
@@ -1212,6 +1250,12 @@ try_next_param:
 		if (negate) {
 			special_variable_characters[0] = '\0';
 		} else {
+			for (i = 0; cp[i]; i++) {
+				if (is_mathomatic_operator(cp[i])) {
+					error(_("Invalid character in list, character is a Mathomatic operator."));
+					return false;
+				}
+			}
 			my_strlcpy(special_variable_characters, cp, sizeof(special_variable_characters));
 		}
 		return true;
@@ -1339,16 +1383,19 @@ try_next_param:
 		quiet_mode = negate;
 		goto try_next_param;
 	}
-#if	!SECURE && !LIBRARY
 	if (strncasecmp(option_string, "html", 4) == 0) {
-		if (security_level) {
+#if	!SECURE
+		if (security_level > 0) {
+#endif
 			error(_("Option disabled by security level."));
 			return false;
+#if	!SECURE
 		}
+#endif
+		reset_attr();
 		html_flag = !negate;
 		goto try_next_param;
 	}
-#endif
 	if (strncasecmp(option_string, "preserve", 8) == 0) {
 		preserve_surds = !negate;
 		goto try_next_param;
@@ -1364,8 +1411,11 @@ try_next_param:
 			i = decstrtol(cp, &cp1);
 			if (cp == cp1 || i < 0 || i > 2) {
 				error(_("Please specify the modulus mode number (0, 1, or 2)."));
-				printf(_("0 means modulus operator (dividend %% divisor) result has same sign as dividend (like C);\n"));
-				printf(_("1 means computed result always has same sign as the divisor (like Python);\n"));
+				printf(_("* C programming language mode 0:\n"));
+				printf(_("0 means modulus operator (dividend %% divisor) result has same sign as dividend;\n"));
+				printf(_("* Python programming language mode 1:\n"));
+				printf(_("1 means computed result always has same sign as the divisor;\n"));
+				printf(_("* Mathematically correct mode 2 for perfect simplification:\n"));
                 		printf(_("2 means the result is always positive or zero.\n"));
 				printf(_("Current value is %d.\n"), modulus_mode);
 				return false;
@@ -1458,6 +1508,8 @@ int
 set_cmd(cp)
 char	*cp;
 {
+	int	rv;
+
 	if (*cp == '\0') {
 		fprintf(gfp, _("Options are set as follows:\n\n"));
 
@@ -1468,7 +1520,16 @@ char	*cp;
 		output_current_directory(gfp);
 		return true;
 	}
-	return set_options(cp);
+	rv = set_options(cp);
+	if (rv) {
+		debug_string(0, _("Success."));
+#if	LIBRARY
+		if (result_str == NULL) {
+			result_str = strdup(_("Success."));
+		}
+#endif
+	}
+	return rv;
 }
 
 /*
@@ -1803,7 +1864,7 @@ char	*cp;
 	int		factor_flag = false, value_entered;
 
 	for (;; cp = skip_param(cp)) {
-		if (strcmp_tospace(cp, "factor") == 0) {
+		if (strcmp_tospace(cp, "factor") == 0 || strcmp_tospace(cp, "factorize") == 0) {
 			factor_flag = true;
 			continue;
 		}
@@ -1812,7 +1873,7 @@ char	*cp;
 	if (!get_range(&cp, &first, &last)) {
 		return false;
 	} 
-	if (first == last && *cp) {
+	if (/* first == last && */ *cp) {
 		cp = parse_var2(&it_v, cp);
 		if (cp == NULL) {
 			return false;
@@ -1828,7 +1889,7 @@ char	*cp;
 			return false;
 		}
 		if (iterations == 0) {
-			warning(_("Calculation will be iterated until convergence."));
+			warning(_("Feedback calculation will be iterated until convergence."));
 			iterations = LONG_MAX - 1L;
 		}
 	}
@@ -1847,8 +1908,8 @@ calc_again:
 		}
 		if (it_v) {
 			if (!found_var(source, n, it_v)) {
-				error(_("Specified iteration variable not found."));
-				return false;
+				debug_string((first == last) ? 0 : 1, _("Specified feedback variable not found."));
+				continue;
 			}
 		}
 		n_trhs = n;
@@ -2344,6 +2405,7 @@ char	*cp;
 			return false;
 		}
 		fprintf(gfp, _("You have entered a base variable.\n"));
+		point_flag = false;
 	}
 	i = next_espace();
 	if (*cp) {
@@ -2374,8 +2436,14 @@ do_repeat_prompt:
 	}
 	fprintf(gfp, "\n");
 /* simplify the operand expressions */
+#if	1
+	simp_loop(rhs[i], &nright);
+	simp_loop(lhs[i], &nleft);
+#else
+/* approximates, too */
 	calc_simp(rhs[i], &nright);
 	calc_simp(lhs[i], &nleft);
+#endif
 /* if division by zero, display a warning */
 	if (get_constant(lhs[i], nleft, &d2)) {
 		check_divide_by_zero(d2);
@@ -2670,11 +2738,11 @@ char	*cp;
 
 	orig_fractions_display_mode = fractions_display;
 	for (;; cp = skip_param(cp)) {
-		if (strcmp_tospace(cp, "factor") == 0) {
+		if (strncasecmp(cp, "factor", 4) == 0) {
 			factor_flag = true;
 			continue;
 		}
-		if (strcmp_tospace(cp, "mixed") == 0) {
+		if (strncasecmp(cp, "mixed", 3) == 0) {
 			mixed_flag = true;
 			continue;
 		}
@@ -2703,7 +2771,7 @@ char	*cp;
 				make_fractions_and_group(i);
 				fractions_display = orig_fractions_display_mode;
 				if (factor_flag || factor_int_flag) {
-					factor_int_sub(i);
+					factor_int_equation(i);
 				}
 				blt(jmp_save, save_save, sizeof(jmp_save));
 #if     LIBRARY
@@ -2860,7 +2928,6 @@ char	*cp;
 			} else if (!solved_equation(k)) {
 				warning(_("Equation is not solved for a normal variable."));
 			}
-			/* approximate_roots = true; No need, user can run the approximate command if desired */
 			simp_i(lhs[k], &n_lhs[k]);
 			if (int_flag) {
 				uf_repeat_always(lhs[k], &n_lhs[k]);
@@ -2871,7 +2938,6 @@ char	*cp;
 					uf_repeat_always(rhs[k], &n_rhs[k]);
 				}
 			}
-			/* approximate_roots = false; */
 			make_fractions_and_group(k);
 			if (int_flag && (!int_expr(lhs[k], n_lhs[k]) || !int_expr(rhs[k], n_rhs[k]))) {
 				warning(_("Not an integer expression."));
@@ -2931,7 +2997,8 @@ char	*cp;
 	int			int_flag = false, imag_flag = false, count_flag = false;
 	char			imag_array[N_EQUATIONS];
 	char			*range_start, *cp1;
-	int			rv = true;
+	int			array_element_flag = false;
+	int			rv;
 
 	CLEAR_ARRAY(imag_array);
 	if (strcmp_tospace(cp, "c") == 0 || strcmp_tospace(cp, "c++") == 0) {
@@ -2948,6 +3015,7 @@ char	*cp;
 		cp = skip_param(cp);
 		count_flag = true;
 	}
+	rv = (lang_code != 0);
 	range_start = cp;
 	do {
 		cp1 = cp;
@@ -3033,17 +3101,26 @@ char	*cp;
 		va[vc].count = cnt;
 		vc++;
 	}
-	if (vc <= 0)
-		return true;
+	if (vc <= 0) {
+		if (lang_code == 0) {
+			error(_("Expression is numeric.  No normal variables found."));
+			return false;
+		} else {
+			return true;
+		}
+	}
 	qsort((char *) va, vc, sizeof(*va), vcmp);
 	for (i1 = 0; i1 < vc; i1++) {
 		if (lang_code && va[i1].v < SIGN) {
 			continue;
 		}
-		list_var(va[i1].v, lang_code);
+		if (lang_code == 0 && (va[i1].v & VAR_MASK) > SIGN) {
+			rv = true;
+		}
+		list_var(va[i1].v, lang_code ? lang_code : -5);
 		if (lang_code) {
-			if (strchr(var_str, '['))
-				rv = false;
+			if (strpbrk(var_str, "[]()"))
+				array_element_flag = true;
 			if (imag_flag) {
 				for (k = 0; k < n_equations; k++) {
 					if (imag_array[k] && n_lhs[k] == 1
@@ -3071,11 +3148,12 @@ char	*cp;
 		printf(_("Manual adjustments may be necessary\n"));
 		printf(_("because of the appearance of the imaginary unit (i).\n"));
 	}
-	if (!rv) {
-		printf("\n");
-		warning(_("Some defined variables were array elements."));
-		printf(_("Arrays require manual definitions,\n"));
-		printf(_("to compile and work properly.\n"));
+	if (lang_code == 0 && !rv) {
+		error(_("Expression is numeric.  No normal variables found."));
+	}
+	if (array_element_flag) {
+		warning(_("Some defined variables were array elements or functions, requiring manual definition."));
+		rv = false;
 	}
 	return rv;
 }
@@ -3398,26 +3476,24 @@ char	*cp;
 {
 	int	i, j;
 	int	i1;
-	int	found;
+	int	found, rv = true;
 	long	v;				/* Mathomatic variable */
 	int	valid_range = false, power_flag = false;
 	char	*cp_start;
-#if	!LIBRARY
 	int	count_down;
 	char	*cp1, *cp2;
 	double	d, ed;
+#if	!LIBRARY
 	char	buf[MAX_CMD_LEN];
 #endif
 
 	cp_start = cp;
-#if	!LIBRARY /* no integer factoring in library mode */
 	if (strcmp_tospace(cp, "number") == 0) {
 		cp = skip_param(cp);
 	} else if (strcmp_tospace(cp, "numbers") == 0) {
 		repeat_flag = true;
 		cp = skip_param(cp);
 	} else {
-#endif
 		if (strcmp_tospace(cp, "power") == 0) {
 			power_flag = true;
 			cp = skip_param(cp);
@@ -3429,24 +3505,32 @@ char	*cp;
 #else		/* be helpful */
 			if (*cp == '-' || isdigit(*cp)) {
 				printf(_("Factoring integers on command line instead:\n"));
+				point_flag = false;
 			} else
 				return false;
 #endif
 		}
-#if	!LIBRARY
 	}
 	if (!valid_range) {
+#if	LIBRARY
+		repeat_flag = false;
+#endif
 		do {
 			if (*cp == '\0') {
 retry:
+#if	LIBRARY
+				return false;
+#else
 				my_strlcpy(prompt_str, _("Enter integers to factor: "), sizeof(prompt_str));
 				cp = get_string(buf, sizeof(buf));
 				if (cp == NULL)
 					return false;
 				cp_start = cp;
+#endif
 			}
 			if (*cp == '\0')
 				return true;
+			rv = true;
 			for (; *cp; ) {
 				cp1 = cp = skip_space(cp);
 				errno = 0;
@@ -3472,7 +3556,7 @@ try_parsing:
 							goto retry;
 						cp_start = cp;
 						if (n_tes <= 0)
-							return true;
+							return rv;
 						calc_simp(tes, &n_tes);
 						if (n_tes != 1 || tes[0].kind != CONSTANT) {
 							error(_("Integer expected."));
@@ -3486,29 +3570,29 @@ try_parsing:
 				for (; count_down ? (d >= ed) : (d <= ed); count_down ? (d -= 1.0) : (d += 1.0)) {
 					if (!factor_one(d)) {
 						error(_("Number too large to factor or not a non-zero integer."));
+						rv = false;
 						break;
 					}
 					display_unique();
 					if (is_prime()) {
-						fprintf(gfp, _("Prime number!\n"));
+						debug_string(0, _("Prime number!"));
 					}
 				}
 			}
 		} while (repeat_flag);
-		return true;
+		return rv;
 	}
-#endif
 	if (power_flag) {
 		if (extra_characters(cp))
 			return false;
 		for (i1 = i; i1 <= j; i1++) {
 			if (n_lhs[i1]) {
-				factor_power(lhs[i1], &n_lhs[i1]);
+/*				factor_power(lhs[i1], &n_lhs[i1]); */
 				do {
 					simp_loop(lhs[i1], &n_lhs[i1]);
 				} while (factor_power(lhs[i1], &n_lhs[i1]));
 				if (n_rhs[i1]) {
-					factor_power(rhs[i1], &n_rhs[i1]);
+/*					factor_power(rhs[i1], &n_rhs[i1]); */
 					do {
 						simp_loop(rhs[i1], &n_rhs[i1]);
 					} while (factor_power(rhs[i1], &n_rhs[i1]));
@@ -3818,19 +3902,19 @@ char	*cp;
 	}
 	rv = read_sub(fp);
 	show_usage = false;
+	if (fclose(fp)) {
+		perror(buf);
+		rv = 1;
+	}
 #if	!SILENT
 	if (rv) {
-		printf(_("Read operation on file \"%s\" aborted.\n"), buf);
+		printf(_("Read operation on file \"%s\" aborted due to error.\n"), buf);
 	} else {
 		if (!quiet_mode && debug_level >= 0) {
-			printf(_("Finished reading file \"%s\".\n"), buf);
+			printf(_("Successfully finished reading file \"%s\".\n"), buf);
 		}
 	}
 #endif
-	if (fclose(fp)) {
-		perror(buf);
-		return false;
-	}
 	return(!rv);
 }
 
