@@ -1,7 +1,7 @@
 /*
  * Mathomatic symbolic factorizing routines, not polynomial factoring.
  *
- * Copyright (C) 1987-2011 George Gesslein II.
+ * Copyright (C) 1987-2012 George Gesslein II.
  
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -17,7 +17,7 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
-The chief copyright holder can be contacted at gesslein@linux.com, or
+The chief copyright holder can be contacted at gesslein@mathomatic.org, or
 George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
@@ -948,6 +948,7 @@ f_again:
 				break;
 		}
 	}
+
 	for (i = loc; i < *np && equation[i].level >= level;) {
 		if (equation[i].level > level) {
 			modified |= fpower_recurse(equation, np, i, level + 1);
@@ -988,6 +989,14 @@ int		*np, loc, i1, n1, i2, n2, level;
 		if (i >= e1)
 			return false;
 		if (equation[i].level == level && equation[i].token.operatr == POWER) {
+#if	1
+/* avoid absolute values: */
+			if (i > (i1 + 2) && equation[i+1].level == level && equation[i-1].level == (level + 1)
+			    && equation[i+1].kind == CONSTANT && equation[i-1].kind == CONSTANT
+			    && equation[i-2].level == (level + 1) && equation[i-2].token.operatr == POWER) {
+				return false;
+			}
+#endif
 			break;
 		}
 	}
@@ -995,10 +1004,19 @@ int		*np, loc, i1, n1, i2, n2, level;
 		if (j >= e2)
 			return false;
 		if (equation[j].level == level && equation[j].token.operatr == POWER) {
+#if	1
+/* avoid absolute values: */
+			if (j > (i2 + 2) && equation[j+1].level == level && equation[j-1].level == (level + 1)
+			    && equation[j+1].kind == CONSTANT && equation[j-1].kind == CONSTANT
+			    && equation[j-2].level == (level + 1) && equation[j-2].token.operatr == POWER) {
+				return false;
+			}
+#endif
 			break;
 		}
 	}
 	start2 = j;
+#if	1
 	if (se_compare(&equation[i+1], e1 - (i + 1), &one_token, 1, &diff_sign)) {
 		return false;
 	}
@@ -1009,6 +1027,7 @@ int		*np, loc, i1, n1, i2, n2, level;
 		j = e2;
 		goto common_exponent;
 	}
+#endif
 
 fp_outer:
 	pop1 = equation[i].token.operatr;
