@@ -1,7 +1,7 @@
 /*
  * Mathomatic floating point constant factorizing routines.
  *
- * Copyright (C) 1987-2011 George Gesslein II.
+ * Copyright (C) 1987-2012 George Gesslein II.
  
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -17,7 +17,7 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
-The chief copyright holder can be contacted at gesslein@linux.com, or
+The chief copyright holder can be contacted at gesslein@mathomatic.org, or
 George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
@@ -257,10 +257,10 @@ int		*np;
  * Factor integers in an equation space.
  */
 void
-factor_int_sub(n)
+factor_int_equation(n)
 int	n;	/* equation space number */
 {
-	if (n_lhs[n] <= 0)
+	if (empty_equation_space(n))
 		return;
 	factor_int(lhs[n], &n_lhs[n]);
 	factor_int(rhs[n], &n_rhs[n]);
@@ -326,7 +326,7 @@ token_type	*equation;
 int		*np, loc, level;
 int		level_code;
 {
-	int	i, j, k;
+	int	i, j, k, eloc;
 	int	op;
 	double	d, minimum = 1.0, cogcd = 1.0;
 	int	improve_readability, gcd_flag, first = true, neg_flag = true, modified = false;
@@ -385,6 +385,11 @@ int		level_code;
 		} else {
 			op = 0;
 			for (j = i + 1; j < *np && equation[j].level > level; j += 2) {
+#if	DEBUG
+				if (equation[j].kind != OPERATOR) {
+					error_bug("Bug in factor_constants().");
+				}
+#endif
 				if (equation[j].level == level + 1) {
 					op = equation[j].token.operatr;
 				}
@@ -433,6 +438,7 @@ int		level_code;
 		}
 		i++;
 	}
+	eloc = i;
 	if (gcd_flag && cogcd != 0.0 /* && fmod(cogcd, 1.0) == 0.0 */) {
 		minimum = cogcd;
 	}
@@ -441,10 +447,10 @@ int		level_code;
 	if (minimum == 0.0 || !isfinite(minimum))
 		return modified;
 	if (improve_readability) {
-		for (i = loc;;) {
+		for (i = loc; i < eloc;) {
 			d = 1.0;
 			if (equation[i].kind == CONSTANT) {
-				if (equation[i].level == level || ((i + 1) < *np
+				if (equation[i].level == level || ((i + 1) < eloc
 				    && equation[i].level == (level + 1)
 				    && equation[i+1].level == (level + 1)
 				    && (equation[i+1].token.operatr == TIMES
