@@ -3,7 +3,7 @@
  *
  * Everything that depends on the command table goes here.
  *
- * Copyright (C) 1987-2011 George Gesslein II.
+ * Copyright (C) 1987-2012 George Gesslein II.
  
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -19,7 +19,7 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
-The chief copyright holder can be contacted at gesslein@linux.com, or
+The chief copyright holder can be contacted at gesslein@mathomatic.org, or
 George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
@@ -65,7 +65,7 @@ static com_type com_list[] = {
 #endif
 {	"eliminate",	NULL,			eliminate_cmd,		"variables or \"all\" [\"using\" equation-number]",	"Substitute the specified variables with solved equations.", "This command may be preceded with \"repeat\"." },
 {	"extrema",	NULL,			extrema_cmd,		"[variable] [order]",					"Show where the slope of the current equation equals zero.", "Helps with finding the minimums and maximums." },
-{	"factor",	"collect",		factor_cmd,		"[\"number\" [integers]] or [\"power\"] [equation-number-range] [variables]",	"Factor variables in equation spaces or factor given integers." },
+{	"factor",	"collect",		factor_cmd,		"[\"number\" [integers]] or [\"power\"] [equation-number-range] [variables]",	"Factor variables in equation spaces or factorize given integers." },
 {	"for",		NULL,			for_cmd,		"variable start end [step-size]",			"Evaluate and display the current expression for each value of variable.", "Same syntax as the sum and product commands." },
 {	"fraction",	NULL,			fraction_cmd,		"[\"numerator\" \"denominator\"] [equation-number-range]",	"Convert expression to a single simple algebraic fraction.", "This command may be preceded with \"repeat\"." },
 #if	HELP
@@ -84,7 +84,7 @@ static com_type com_list[] = {
 #endif
 {	"product",	NULL,			product_cmd,		"variable start end [step-size]",			"Compute the product as variable goes from start to end.", "Related command: sum" },
 #if	READLINE
-{	"push",		NULL,			push_cmd,		"[equation-number-ranges]",				"Push equation spaces into the readline history for editing.", "Available only if readline is enabled." },
+{	"push",		NULL,			push_cmd,		"[equation-number-ranges or text-to-push]",		"Push equation spaces or text into readline history for editing.", "Available only if readline is enabled." },
 #endif
 #if	!LIBRARY
 {	"quit",		"exit",			quit_cmd,		"[exit-value]",						"Terminate this program without saving." },
@@ -103,7 +103,7 @@ static com_type com_list[] = {
 {	"solve",	NULL,			solve_cmd,		"[\"verify\"] [equation-number-range] [\"for\"] variable or \"0\"",	"Solve the specified equations for a variable or for zero.", "The verify option checks all displayed solutions." },
 {	"sum",		NULL,			sum_cmd,		"variable start end [step-size]",			"Compute the summation as variable goes from start to end.", "Related command: product" },
 #if	!LIBRARY
-{	"tally",	NULL,			tally_cmd,		"[\"average\"]",					"Prompt for and add entries, showing running total and average." },
+{	"tally",	NULL,			tally_cmd,		"[\"average\"]",					"Prompt for and add entries, showing running total." },
 #endif
 {	"taylor",	NULL,			taylor_cmd,		"[\"nosimplify\"] variable order point",		"Compute the Taylor series expansion of the current expression." },
 {	"unfactor",	"expand",		unfactor_cmd,		"[\"count\" \"fraction\" \"quick\" \"power\"] [equation-number-range]",	"Algebraically expand (multiply out) expressions." },
@@ -122,7 +122,7 @@ char	*example_strings[] = {
 	"repeat echo -\n",
 	"; Example 2:\n",
 	"; Here the calculate command is used to plug values into a solved formula.\n",
-	"; A common temperature conversion formula:\n",
+	"; A common temperature conversion formula (from \"help conversions\"):\n",
 	"fahrenheit = (9*celsius/5) + 32\n",
 	"repeat calculate ; plug in values until an empty line is entered\n",
 	"\n",
@@ -136,10 +136,60 @@ char	*example_strings[] = {
 	"repeat echo -\n",
 	"; Example 3:\n",
 	"; Expand the following to polynomial form, then refactor and differentiate:\n",
-	"(x+1)^6\n",
+	"(x-y)^6\n",
 	"expand count ; Expand and count the resulting number of terms:\n",
 	"simplify\n",
-	"derivative\n",
+	"derivative x\n",
+	NULL
+};
+#endif
+
+#if	HELP
+char	*geometry_strings[] = {
+	"; Triangle area:\n",
+	"triangle_area = base*height/2\n",
+	"; See Heron's formula for the area of any triangle\n",
+	"; given all three side lengths.\n",
+	"\n",
+	"; Rectangle of length \"l\" and width \"w\":\n",
+	"rectangle_area = l*w\n",
+	"rectangle_perimeter = 2*l + 2*w\n",
+	"\n",
+	"; Trapezoid of parallel sides \"a\" and \"b\",\n",
+	"; and the \"distance\" between them:\n",
+	"trapezoid_area = distance*(a + b)/2\n",
+	"\n",
+	"; Circle of radius \"r\":\n",
+	"circle_area = pi*r^2\n",
+	"circle_perimeter = 2*pi*r\n",
+	"\n",
+	"; 3D rectangular solid of length \"l\", width \"w\", and height \"h\":\n",
+	"brick_volume = l*w*h\n",
+	"brick_surface_area = 2*l*w + 2*l*h + 2*w*h\n",
+	"\n",
+	"; 3D sphere of radius \"r\":\n",
+	"sphere_volume = 4/3*pi*r^3\n",
+	"sphere_surface_area = 4*pi*r^2\n",
+	"\n",
+	"; Convex 2D polygon with straight sides,\n",
+	"; sum of all interior angles formula:\n",
+	"sum_degrees = (sides - 2)*180\n",
+	"sum_radians = (sides - 2)*pi\n",
+	"; \"sides\" is the number of sides of any convex 2D polygon.\n",
+	"; Convex means that all interior angles are less than 180 degrees.\n",
+	"; Type \"eliminate sides\" to get radians/degrees conversion formula.\n",
+	NULL
+};
+
+char	*conversion_strings[] = {
+	"; Temperature\n",
+	"fahrenheit = (9*celsius/5) + 32\n",
+	"kelvin = celsius + 273.15\n",
+	"; Distance\n",
+	"inches = centimeters/2.54\n",
+	"miles = kilometers/1.609344\n",
+	"; Weight\n",
+	"pounds = kilograms/0.45359237\n",
 	NULL
 };
 #endif
@@ -326,12 +376,16 @@ char	*cp;
 	int	rv;
 	char	buf2[MAX_CMD_LEN];	/* do not make this static! */
 	int	our_repeat_flag = false;
+	long	v;			/* Mathomatic variable */
 #if	!SECURE
 	FILE	*fp;
 	int	append_flag;
 	int	i1;
 #endif
 
+#if	DEBUG
+	check_gvars();
+#endif
 	init_gvars();		/* make sure we are in the default state */
 	set_sign_array();	/* register all sign variables so that the next ones will be unique */
 	if (cp == NULL) {
@@ -339,9 +393,33 @@ char	*cp;
 	}
 	cp_start = cp;
 	cp = skip_space(cp);
-/* handle the equation number selector */
+/* handle search forward */
+	if (*cp == '/' && isvarchar(cp[1])) {
+		cp++;
+		if (parse_var(&v, cp) == NULL) {
+			return false;
+		}
+		if (search_all_for_var(v, true)) {
+			return return_result(cur_equation);
+		} else {
+			put_up_arrow(cp - cp_start, _("Variable not found in any equation space."));
+			return false;
+		}
+	}
+/* handle the equation selector */
 	if (*cp == '#') {
 		cp++;
+		if (isvarchar(*cp)) {
+			if (parse_var(&v, cp) == NULL) {
+				return false;
+			}
+			if (search_all_for_var(v, false)) {
+				return return_result(cur_equation);
+			} else {
+				put_up_arrow(cp - cp_start, _("Variable not found in any equation space."));
+				return false;
+			}
+		}
 		switch (*cp) {
 		case '+':
 		case '-':
@@ -671,7 +749,6 @@ display_repeat_command(void)
 	return 3;
 }
 
-#if	!LIBRARY
 int
 read_examples(cpp)
 char	**cpp;
@@ -691,7 +768,6 @@ char	**cpp;
 	}
 	return true;
 }
-#endif
 
 /*
  * Display a row of dashes to underline a title.
@@ -730,7 +806,7 @@ intro:
 		P("Type \"help equations\" for help with entering expressions and equations.");
 		P("Type \"help all\" for a summary and syntax of all commands.");
 		P("Type \"help usage\" to display the syntax of all commands.");
-		P("Other help keywords: constants, color, license, bugs, geometry.");
+		P("Other help topics: constants, color, license, bugs, geometry, conversions.");
 		P("\"help\" or \"?\" followed by a command name will give info on that command.");
 		fprintf(gfp, "These are the %d commands for this version of Mathomatic:\n", ARR_CNT(com_list));
 		set_color(0);
@@ -743,9 +819,8 @@ intro:
 		}
 		default_color();
 
-		P("\n\nTo select an equation space, type the equation number at the main prompt.");
-		P("To solve the current equation, type the variable name at the main prompt.");
-		P("For more help, go to the website: www.mathomatic.org");
+		P("\n\nTo see what is allowed at the main prompt, type \"help main\".");
+		P("For more help, go to the official website: www.mathomatic.org");
 	} else if (strcasecmp(cp, "table") == 0) {
 		fprintf(gfp, "Mathomatic version %s Quick Reference Card\n", VERSION);
 		fprintf(gfp, "Command\tUsage\tNotes\n");
@@ -769,7 +844,11 @@ next_argument:
 	while (*cp1 && !isspace(*cp1))
 		cp1++;
 	if (cp1 != cp) {
+#if	LIBRARY
+		if (html_flag) {
+#else
 		if (html_flag && gfp == stdout) {
+#endif
 			P("<hr>");
 		} else {
 			P("*******************************************************************************");
@@ -807,6 +886,45 @@ next_argument:
 			return read_examples(example_strings);
 		}
 #endif
+		if (strncasecmp(cp, "geometry", cp1 - cp) == 0) {
+			underline_title(P("Commonly used standard (Euclidean) geometric formulas"));
+			return read_examples(geometry_strings);
+		}
+		if (strncasecmp(cp, "conversions", cp1 - cp) == 0) {
+			P("Commonly used metric/English conversions.");
+			P("Select the equation you want and type the unit name you want, to solve for it.");
+			P("Then type \"repeat calculate\" for units conversion and trying different values.");
+			underline_title(P("These values are correct for the US and UK."));
+			return read_examples(conversion_strings);
+		}
+		if (strncasecmp(cp, "main", cp1 - cp) == 0 || strncasecmp(cp, "prompt", cp1 - cp) == 0) {
+			P("At the Mathomatic main prompt, you may enter:\n");
+
+			P("  * a numerical expression, which is instantly evaluated and displayed with");
+			P("    the calculate command (autocalc) (see \"help constants\"),");
+			P("  * an algebraic expression or equation, which is stored and made the current");
+			P("    equation (see \"help expressions\"),");
+			P("  * a variable to solve the current equation for (autosolve),");
+			P("  * an equation number to select as the current equation (autoselect),");
+			P("  * a slash (/) or pound sign (#) followed by a variable name to search all");
+			P("    equations spaces forward or backward for,");
+			P("  * a Mathomatic command (see \"help all\"),");
+			P("  * a question mark (?) for quick, short help (same as the help command),");
+			P("  * a semicolon (;) followed by a comment (everything on a line after a");
+			P("    semicolon is ignored),");
+#if	SHELL_OUT
+			P("  * or an exclamation point (!) followed by a shell or system command. \"!\" by");
+			P("    itself invokes the default shell.\n");
+#else
+			P("  * shelling out (!) is disabled by security level or system type.\n");
+#endif
+
+			P("If a colon (:) starts the line, preceding any of the above input to the main");
+			P("prompt, it will always return with successful status, preventing any current");
+			P("read command operations from aborting.");
+
+			goto next_space;
+		}
 		if (strncasecmp(cp, "copyrights", cp1 - cp) == 0 || strncasecmp(cp, "licenses", cp1 - cp) == 0) {
 			underline_title(P("Copyright and License for Mathomatic"));
 
@@ -845,30 +963,6 @@ next_argument:
 			}
 			goto next_space;
 		}
-		if (strncasecmp(cp, "geometry", cp1 - cp) == 0) {
-			underline_title(P("Commonly used standard (Euclidean) geometric formulas"));
-			P("Triangle of base \"b\" and height \"h\":");
-			P("    area = b*h/2\n");
-
-			P("Rectangle of length \"l\" and width \"w\":");
-			P("    area = l*w                    perimeter = 2*l + 2*w\n");
-
-			P("Trapezoid of parallel sides \"a\" and \"b\", and \"d\" distance between them:");
-			P("    area = d*(a + b)/2\n");
-
-			P("Circle of radius \"r\":");
-			P("    area = pi*r^2                 perimeter = 2*pi*r\n");
-
-			P("Rectangular solid of length \"l\", width \"w\", and height \"h\":");
-			P("    volume = l*w*h                surface_area = 2*l*w + 2*l*h + 2*w*h\n");
-
-			P("Sphere of radius \"r\":");
-			P("    volume = 4/3*pi*r^3           surface_area = 4*pi*r^2\n");
-
-			P("Convex polygon of \"n\" sides, sum of all interior angles formula:");
-			P("    sum = (n - 2)*180 degrees     sum = (n - 2)*pi radians");
-			goto next_space;
-		}
 		if (cp[0] == '!') {
 			P("A command preceded by an exclamation point (such as \"!ls\") is taken to");
 			P("be a shell command and is passed unchanged to the shell (/bin/sh) when");
@@ -883,8 +977,8 @@ next_argument:
 			P("To enter an expression or equation, simply type it in at the prompt.");
 			P("Operators have precedence decreasing as indicated:\n");
 
-			P("    ! factorial (also gamma() function, highest precedence)");
-			P("    ** or ^ power (exponentiation)");
+			P("    ! factorial (same as gamma(x+1) function; highest precedence)");
+			P("    ** or ^ power (exponentiation; high precedence)");
 			P("    * multiply      / divide        % modulus       // integral divide");
 			P("    + add           - subtract or negate");
 			P("    = equate (lowest precedence)\n");
@@ -946,7 +1040,7 @@ next_argument:
 				if (display_all_colors()) {
 					fprintf(gfp, " are the available colors.\n");
 				} else {
-					fprintf(gfp, "Can't show available colors.\n");
+					fprintf(gfp, "\n");
 				}
 			} else {
 				fprintf(gfp, "Color mode is currently turned off.\n");
