@@ -2,7 +2,7 @@
  * Mathomatic expression and equation display routines.
  * Color mode routines, too.
  *
- * Copyright (C) 1987-2011 George Gesslein II.
+ * Copyright (C) 1987-2012 George Gesslein II.
  
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
-The chief copyright holder can be contacted at gesslein@linux.com, or
+The chief copyright holder can be contacted at gesslein@mathomatic.org, or
 George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
@@ -100,18 +100,25 @@ static char	*html_carray[] = {
 void
 reset_attr(void)
 {
+FILE	*fp;
+
+#if	LIBRARY
+	fp = gfp;
+#else
+	fp = stdout;
+#endif
 	fflush(NULL);	/* flush all output */
 	if (color_flag && cur_color >= 0) {
 		if (html_flag) {
-			printf("</font>");
+			fprintf(fp, "</font>");
 		} else {
 #if	WIN32_CONSOLE_COLORS
                         SetConsoleTextAttribute(hOut, FOREGROUND_WHITE);
 #else
-                        printf("\033[0m");
+                        fprintf(fp, "\033[0m");
 #endif
 		}
-		fflush(stdout);	/* flush standard output */
+		fflush(fp);
 	}
 	cur_color = -1;
 }
@@ -130,19 +137,21 @@ int	color;
 {
 	int	rv = -1;
 
+#if	!LIBRARY
 	if (gfp != stdout)
 		return rv;
-	if (cur_color == color)
-		return rv;
+#endif
 	if (color_flag) {
+		if (cur_color == color)	/* color already set */
+			return rv;
 		if (html_flag) {
 			if (cur_color >= 0) {
-				printf("</font>");
+				fprintf(gfp, "</font>");
 			}
 			if (bold_colors) {
-				printf("<font color=\"%s\">", bright_html_carray[rv = (color % ARR_CNT(bright_html_carray))]);
+				fprintf(gfp, "<font color=\"%s\">", bright_html_carray[rv = (color % ARR_CNT(bright_html_carray))]);
 			} else {
-				printf("<font color=\"%s\">", html_carray[rv = (color % ARR_CNT(html_carray))]);
+				fprintf(gfp, "<font color=\"%s\">", html_carray[rv = (color % ARR_CNT(html_carray))]);
 			}
 		} else {
 #if	WIN32_CONSOLE_COLORS
@@ -151,7 +160,7 @@ int	color;
                                  attr |= FOREGROUND_INTENSITY;
                         SetConsoleTextAttribute(hOut, attr);
 #else
-                        printf("\033[%d;%dm", bold_colors, carray[rv = (color % ARR_CNT(carray))]);
+                        fprintf(gfp, "\033[%d;%dm", bold_colors, carray[rv = (color % ARR_CNT(carray))]);
 #endif
 		}
 		cur_color = color;
@@ -165,21 +174,24 @@ int	color;
 void
 default_color(void)
 {
-	if (gfp == stdout) {
-		if (color_flag && cur_color >= 0) {
-			if (html_flag) {
-				printf("</font>");
-			} else {
-#if	WIN32_CONSOLE_COLORS
-                                SetConsoleTextAttribute(hOut, FOREGROUND_WHITE);
-#else
-                                printf("\033[0m");
-#endif
-			}
-		}
-		cur_color = -1;
+#if	!LIBRARY
+	if (gfp != stdout) {
+		return;
 	}
-	fflush(stdout);
+#endif
+	if (color_flag && cur_color >= 0) {
+		if (html_flag) {
+			fprintf(gfp, "</font>");
+		} else {
+#if	WIN32_CONSOLE_COLORS
+			SetConsoleTextAttribute(hOut, FOREGROUND_WHITE);
+#else
+			fprintf(gfp, "\033[0m");
+#endif
+		}
+	}
+	cur_color = -1;
+	fflush(gfp);
 }
 
 /*
@@ -239,7 +251,7 @@ int	export_flag;	/* non-zero for exportable format (readable by other math progr
 {
 	int	len = 0;
 
-	if (n_lhs[n] <= 0)
+	if (empty_equation_space(n))
 		return 0;
 	if ((export_flag == 0 || export_flag == 4) && !high_prec) {
 		len += fprintf(gfp, "#%d: ", n + 1);
@@ -270,11 +282,11 @@ int
 list_sub(n)
 int	n;	/* equation space number */
 {
-	if (n_lhs[n] <= 0)
+	if (empty_equation_space(n))
 		return 0;
 	make_fractions_and_group(n);
 	if (factor_int_flag) {
-		factor_int_sub(n);
+		factor_int_equation(n);
 	}
 	if (display2d) {
 		/* display in fraction format */
@@ -335,7 +347,8 @@ long	v;	/* Mathomatic variable */
  *
  * If (lang_code == 0), use standard Mathomatic format.
  * If (lang_code > 0), make variable compatible with the language defined in the enumeration language_list defined in am.h
- * If (lang_code < 0), create an exportable variable name: -1 for Maxima, -2 for other, -3 for gnuplot, -4 for hexadecimal.
+ * If (lang_code < 0), create an exportable variable name: -1 for Maxima, -2 for other, -3 for gnuplot, -4 for hexadecimal,
+ * -5 for mathomatic-only variable format.
  */
 int
 list_var(v, lang_code)
@@ -363,6 +376,9 @@ int	lang_code;	/* language code */
 		case -2:
 			cp = "i";
 			break;
+		case -5:
+			cp = "i#";
+			break;
 		case -1:
 			cp = "%i";
 			break;
@@ -391,6 +407,9 @@ int	lang_code;	/* language code */
 		case PYTHON:
 			cp = "math.e";
 			break;
+		case -5:
+			cp = "e#";
+			break;
 		default:
 			cp = "e";
 			break;
@@ -401,6 +420,9 @@ int	lang_code;	/* language code */
 		case -1:
 			cp = "%pi";
 			break;
+		case -5:
+			cp ="pi#";
+			break;
 		case C:
 			cp = "M_PI";
 			break;
@@ -434,11 +456,19 @@ int	lang_code;	/* language code */
 		my_strlcpy(var_str, "bad_variable", sizeof(var_str));
 	}
 /* Make valid C type variable if exporting or generating code: */
-	if (from_memory && lang_code != 0 && lang_code != -4) {
-		for (j = 0; var_str[j] && var_str[j] != '('; j++) {
-			if (strchr("_[]", var_str[j]) == NULL && !isalnum(var_str[j])) {
-				var_str[j] = '_';
+	if (from_memory) {
+		switch (lang_code) {
+		case 0:
+		case -4:
+		case -5:
+			break;
+		default:
+			for (j = 0; var_str[j] && var_str[j] != '('; j++) {
+				if (strchr("_[]", var_str[j]) == NULL && !isalnum(var_str[j])) {
+					var_str[j] = '_';
+				}
 			}
+			break;
 		}
 	}
 	return(strlen(var_str));
@@ -474,9 +504,8 @@ int	export_flag;	/* flag for exportable format (usually false) */
 	int	len;
 	char	*cp;
 
-	if (n_lhs[n] <= 0) {
+	if (empty_equation_space(n))
 		return NULL;
-	}
 	len = list_string(lhs[n], n_lhs[n], NULL, export_flag);
 	if (n_rhs[n]) {
 		len += strlen(EQUATE_STRING);
@@ -740,7 +769,7 @@ int			int_flag;	/* integer arithmetic flag */
 {
 	int	len = 0;
 
-	if (n_lhs[en] <= 0)
+	if (empty_equation_space(en))
 		return 0;
 	len += list_code(lhs[en], &n_lhs[en], true, NULL, language, int_flag);
 	if (n_rhs[en]) {
@@ -774,9 +803,8 @@ int			int_flag;	/* integer arithmetic flag */
 	int	len;
 	char	*cp;
 
-	if (n_lhs[n] <= 0) {
+	if (empty_equation_space(n))
 		return NULL;
-	}
 	len = list_code(lhs[n], &n_lhs[n], false, NULL, language, int_flag);
 	if (n_rhs[n]) {
 		len += strlen(EQUATE_STRING);
@@ -979,7 +1007,7 @@ int	n;	/* equation space number */
 	int	screen_line;
 	char	*cp;
 
-	if (n_lhs[n] <= 0)
+	if (empty_equation_space(n))
 		return NULL;
 	if (!malloc_vscreen())
 		return NULL;
@@ -1053,9 +1081,9 @@ int	n;	/* equation space number */
 	int	high = 0, low = 0;
 	int	max_line = 0, min_line = 0;
 	int	max2_line = 0, min2_line = 0;
-	int	use_screen_columns;
+	int	use_screen_columns;	/* use infinite width if false */
 
-	if (n_lhs[n] <= 0)
+	if (empty_equation_space(n))
 		return 0;
 #if	ROBOT_COMMAND
 	use_screen_columns = true;
@@ -1140,7 +1168,8 @@ make_smaller:
  * 0 is the middle line, lines above are positive, lines below are negative.
  * Use color if available.
  *
- * The following functions are for internal use only, and not to be called, except by flist_equation().
+ * The following functions are for internal use only, and not to be called,
+ * except by flist_equation() and flist_equation_string().
  *
  * Return the width of the expression (that is, the required number of screen columns).
  */
@@ -1200,10 +1229,12 @@ int		*highp, *lowp;
 	}
 	oflag = (out_flag && line == cur_line);
 	cflag = (oflag && string == NULL);
+#if	LIBRARY
+	html_out = html_flag;
+#else
 	html_out = (html_flag && gfp == stdout);
+#endif
 	if (oflag) {
-		if (cflag)
-			set_color(cur_level-1);
 		for (; cur_pos < pos; cur_pos++) {
 			APPEND2(" ");
 		}
@@ -1262,6 +1293,10 @@ check_again:
 			else
 				k = abs(j);
 		}
+		if (k < 1) {
+			if (cflag)
+				set_color(cur_level-1);
+		}
 		for (i1 = 1; i1 <= k; i1++) {
 			if (j > 0) {
 				cur_level--;
