@@ -18,7 +18,7 @@
  * They are not separated out from the main variable of the polynomial.
  * Maximum simplification of all expressions is not possible unless everything is generalized.
  *
- * Copyright (C) 1987-2011 George Gesslein II.
+ * Copyright (C) 1987-2012 George Gesslein II.
  
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -34,7 +34,7 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
-The chief copyright holder can be contacted at gesslein@linux.com, or
+The chief copyright holder can be contacted at gesslein@mathomatic.org, or
 George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
@@ -311,7 +311,7 @@ int		do_repeat;	/* factor repeated factors flag */
 /* Find a valid polynomial base variable "v": */
 		cnt = -1;
 		if (v) {
-			if (!poly_in_v(trhs, n_trhs, v, true)) {
+			if (vc > 1 && !poly_in_v(trhs, n_trhs, v, true)) {
 				v = 0;
 			}
 		}
@@ -489,6 +489,16 @@ skip_factor:
 			save_factors(equation, np, loc, len_first, level);
 		}
 	}
+	if (modified || symbolic_modified) {
+		for (i = loc; i < *np && equation[i].level >= level; i++)
+			;
+#if	DEBUG
+		if ((i & 1) != 1) {
+			error_bug("Error in result of pf_sub().");
+		}
+#endif
+		side_debug(1, &equation[loc], i - loc);
+	}
 	return(modified || symbolic_modified);
 }
 
@@ -501,9 +511,9 @@ int		*np, loc1, len, level;
 
 	i = n_tlhs + 1 + n_trhs;
 	if (i > (len * 3))
-		return false;
+		goto rejected;
 	if ((*np + (i - len)) > n_tokens)
-		return false;
+		goto rejected;
 	blt(&equation[loc1+i], &equation[loc1+len], (*np - (loc1 + len)) * sizeof(token_type));
 	*np += i - len;
 	blt(&equation[loc1], tlhs, n_tlhs * sizeof(token_type));
@@ -517,6 +527,9 @@ int		*np, loc1, len, level;
 	for (j = loc1; j < i; j++)
 		equation[j].level += level;
 	return true;
+rejected:
+	debug_string(1, "Polynomial factor rejected because too large.");
+	return false;
 }
 
 /*
@@ -533,7 +546,7 @@ remove_factors(void)
 	int	plus_flag = false, divide_flag = false;
 	int	op;
 
-	debug_string(3, "Entering remove_factors() with:");
+	debug_string(3, "Entering remove_factors() with: ");
 	side_debug(3, tlhs, n_tlhs);
 	do {
 		simp_ssub(tlhs, &n_tlhs, 0L, 1.0, false, true, 4);
@@ -546,10 +559,13 @@ remove_factors(void)
 				blt(&scratch[k], &tlhs[j], (i - j) * sizeof(token_type));
 				k += i - j;
 			}
-			if (k <= 0)
+			if (k <= 0) {
+				debug_string(3, "Leaving remove_factors() with false return and no change.");
 				return false;
+			}
 			blt(tlhs, scratch, k * sizeof(token_type));
 			n_tlhs = k;
+			debug_string(3, "Leaving remove_factors() with success and: ");
 			side_debug(3, tlhs, n_tlhs);
 			return true;
 		}
@@ -565,6 +581,7 @@ remove_factors(void)
 			case DIVIDE:
 				break;
 			default:
+				debug_string(3, "Leaving remove_factors() with false return and no change.");
 				return false;
 			}
 			if (plus_flag && !divide_flag) {
@@ -1876,7 +1893,7 @@ int		*tp1, *lentp1;
 }
 
 /*
- * This routine automatically finds the best normal variable to do polynomial division with.
+ * This routine automatically finds the best variable to do polynomial division with.
  *
  * Returns 0 if nothing was found,
  * otherwise the selected polynomial base variable is returned in *vp1,
@@ -1892,7 +1909,7 @@ long		*vp1;		/* variable pointer to return base variable with */
 {
 	int		i;
 	int		vc, cnt;
-	long		v1, last_v;
+	long		v1, last_v, cv;		/* Mathomatic variables */
 	sort_type	va[MAX_VARS];
 	int		divide_flag;
 	int		t1, len_t1;
@@ -1924,23 +1941,28 @@ long		*vp1;		/* variable pointer to return base variable with */
 	if (vc <= 0)
 		return 0;
 	qsort((char *) va, vc, sizeof(*va), vcmp);
-	for (i = 0; i < vc; i++) {
-		if ((va[i].v & VAR_MASK) <= SIGN) {
-			continue;
-		}
-		*vp1 = va[i].v;
-		divide_flag = 2;
-		count1 = find_greatest_power(p1, n1, vp1, &d1, &t1, &len_t1, &divide_flag);
-		count2 = find_greatest_power(p2, n2, vp1, &d2, &t2, &len_t2, &divide_flag);
-		if (d2 <= 0 || d1 < d2 || count2 > count1) {
-			divide_flag = !divide_flag;
+	for (cv = IMAGINARY + 1; cv > 0; cv--) {
+		for (i = 0; i < vc; i++) {
+			if ((cv > IMAGINARY) ? ((va[i].v & VAR_MASK) <= SIGN) : (va[i].v != cv)) {
+				continue;
+			}
+			*vp1 = va[i].v;
+			divide_flag = 2;
 			count1 = find_greatest_power(p1, n1, vp1, &d1, &t1, &len_t1, &divide_flag);
 			count2 = find_greatest_power(p2, n2, vp1, &d2, &t2, &len_t2, &divide_flag);
 			if (d2 <= 0 || d1 < d2 || count2 > count1) {
-				continue;
+				divide_flag = !divide_flag;
+				count1 = find_greatest_power(p1, n1, vp1, &d1, &t1, &len_t1, &divide_flag);
+				count2 = find_greatest_power(p2, n2, vp1, &d2, &t2, &len_t2, &divide_flag);
+				if (d2 <= 0 || d1 < d2 || count2 > count1) {
+					continue;
+				}
 			}
+			return va[i].count;
 		}
-		return va[i].count;
+#if	1
+		break;
+#endif
 	}
 	return 0;
 }
