@@ -1,7 +1,7 @@
 /*
  * Mathomatic symbolic solve routines.
  *
- * Copyright (C) 1987-2011 George Gesslein II.
+ * Copyright (C) 1987-2012 George Gesslein II.
  
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -17,7 +17,7 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
-The chief copyright holder can be contacted at gesslein@linux.com, or
+The chief copyright holder can be contacted at gesslein@mathomatic.org, or
 George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
@@ -61,7 +61,7 @@ int	have;	/* equation number to solve */
 		if (n_lhs[want]) {
 			if (n_rhs[want]) {
 				/* Something in the LHS and RHS of equation number "want". */
-				error(_("This program will only solve for a single variable or for zero."));
+				error(_("Can only solve for a single variable or for 0, possibly raised to a power."));
 				rv = 0;
 			} else {
 				/* Normal solve: */
@@ -150,13 +150,8 @@ int		*rightnp;	/* pointer to length of RHS */
 			n_trhs = *rightnp;
 			blt(trhs, rightp, n_trhs * sizeof(*rightp));
 			uf_simp(tlhs, &n_tlhs);
-			if ((i = increase(1 / wantp[2].token.constant, v)) != true) {
+			if (increase(1 / wantp[2].token.constant, v) != true) {
 				error(_("Unable to isolate root."));
-#if	!SILENT
-				if (i == 2) {
-					printf(_("No matching root found.\n"));
-				}
-#endif
 				return false;
 			}
 			list_tdebug(2);
@@ -171,7 +166,7 @@ int		*rightnp;	/* pointer to length of RHS */
 			*rightnp = n_trhs;
 			return true;
 		}
-		error(_("This program will only solve for a single variable or for zero."));
+		error(_("Can only solve for a single variable or for 0, possibly raised to a power."));
 		return false;
 	}
 	/* copy the equation to temporary storage where it will be manipulated */
@@ -191,7 +186,7 @@ int		*rightnp;	/* pointer to length of RHS */
 	} else {
 		v = 0;
 		if (wantp->kind != CONSTANT || wantp->token.constant != 0.0) {
-			error(_("This program will only solve for a single variable or for zero."));
+			error(_("Can only solve for a single variable or for 0, possibly raised to a power."));
 			return false;
 		}
 		debug_string(1, _("Solving for zero..."));
